import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:args/args.dart';
import 'package:puma/puma.dart';

import 'package:yaml/yaml.dart' as yaml;

Future<void> main(List<String> args) async {
  final parser = initParser();
  final allCommands = [
    PubGetCommand(args),
    PackageCommand(args),
    VersionCommand(args),
    FlutterCommand(args),
    HelpCommand([], args),
  ];
  final helpCommand = HelpCommand(allCommands, args);
  try {
    final results = parser.parse(args);
    final globalArgs = parseGlobalArgs(args);
    final pubspecFile = File('pubspec.yaml');
    final pubspecContent = await pubspecFile.readAsString();
    final yamlRoot = yaml.loadYaml(pubspecContent) as yaml.YamlMap;
    final puma = Puma(
      globalArgs.pubHostUrl,
      Pubspec.fromJson(
        (json.decode(json.encode(yamlRoot)) as Map).cast<String, dynamic>(),
      ),
    );
    final commands = Commands(
      allCommands,
      helpCommand,
    );
    final result = await commands.run(results, puma);
    if (result != null) {
      final pubspecContent = await writeContent(result.toYaml());
      await pubspecFile.writeAsString(pubspecContent, mode: FileMode.writeOnly);
    }
  } on ShowUsageException catch (e) {
    print(e.usage);
  } on ArgParserException catch (e) {
    final command = await getCommand(e.commands, allCommands) ?? helpCommand;
    catchShowUsageException(() async {
      await command.run(null, null);
    });
  } catch (e, stacktrace) {
    print(e);
    print(stacktrace);
    catchShowUsageException(() async {
      print(await helpCommand.run(null, null));
    });
  }
}
