enum PackageType {
  hosted,
  git,
  path,
}

abstract class PackageDetails {
  static PackageType getType(Map<String, dynamic> json) {
    final firstValue = json.values.first;
    if (firstValue is String) {
      return PackageType.hosted;
    } else if (json.values.first is Map) {
      if (firstValue is Map && firstValue.keys.contains('path')) {
        return PackageType.path;
      }
    }
    return null;
  }

  final String name;

  const PackageDetails(this.name);

  factory PackageDetails.fromJson(Map<String, dynamic> json) {
    final type = getType(json);
    if (type == PackageType.hosted) {
      return HostedPackage.fromJson(json);
    } else if (type == PackageType.path) {
      return PathPackage.fromJson(json);
    }
    //  TODO: add git dependencies
    throw 'Unknown package type';
  }

  MapEntry<String, dynamic> toMapEntry();
}

class HostedPackage extends PackageDetails {
  final String version;

  const HostedPackage(String name, this.version) : super(name);

  factory HostedPackage.fromJson(Map<String, dynamic> json) =>
      HostedPackage(json.keys.first, json.values.first);

  MapEntry<String, dynamic> toMapEntry() => MapEntry(name, version);
}

class PathPackage extends PackageDetails {
  final String path;

  const PathPackage(String name, this.path) : super(name);

  factory PathPackage.fromJson(Map<String, dynamic> json) {
    final name = json.keys.first;
    final path = (json.values.first as Map)['path'];
    return PathPackage(name, path);
  }

  MapEntry<String, dynamic> toMapEntry() => MapEntry(name, {'path': path});
}
